<?php

class DLM_CA_Settings {

	/**
	 * Add settings to DLM settings
	 *
	 * @param $settings
	 *
	 * @return mixed
	 */
	public static function add_settings( $settings ) {

		$doc_link = "https://www.download-monitor.com/kb/captcha/";

		$settings['captcha'] = array(
			__( 'Captcha', 'dlm-captcha' ),
			array(
				array(
					'name'    => 'dlm_ca_sitekey',
					'std'     => '',
					'label'   => __( 'reCAPTCHA Site Key', 'dlm-captcha' ),
					'desc'    => sprintf( __( 'The Google reCAPTCHA Site Key can be found in your Google reCAPTCHA dashboard. %s for more information on how to set this up.', 'dlm-captcha' ), '<a href="' . $doc_link . '" target="_blank">'. __( 'Please read our documentation', 'dlm-captcha' ).'</a>' ),
					'type'    => 'text'
				),
				array(
					'name'    => 'dlm_ca_sitesecret',
					'std'     => '',
					'label'   => __( 'reCAPTCHA Secret Key', 'dlm-captcha' ),
					'desc'    => sprintf( __( 'The Google reCAPTCHA Secret Key can be found in your Google reCAPTCHA dashboard. %s for more information on how to set this up.', 'dlm-captcha' ), '<a href="' . $doc_link . '" target="_blank">'. __( 'Please read our documentation', 'dlm-captcha' ).'</a>' ),
					'type'    => 'text'
				),
				array(
					'name'    		=> 'dlm_ca_unlock_text',
					'std'     		=> __( 'Please complete the captcha to download the file.', 'dlm-captcha' ),
					'placeholder' => __( 'Please complete the captcha to download the file.', 'dlm-captcha' ),
					'label'   		=> __( 'Captcha Unlock Text', 'dlm-captcha' ),
					'desc'    		=> __( 'The text displayed above your captcha field.', 'dlm-captcha' ),
					'type'    		=> 'textarea'
				),
				array(
					'name'     => 'dlm_ca_global',
					'std'      => '',
					'label'    => __( 'Global mode', 'dlm-captcha' ),
					'cb_label' => __( 'Enable', 'dlm-captcha' ),
					'desc'     => __( sprintf( 'If enabled, %sALL%s your downloads will be captcha locked.', '<strong>', '</strong>' ), 'dlm-captcha' ),
					'type'     => 'checkbox'
				),
				array(
					'name'     => 'dlm_ca_debug',
					'std'      => '',
					'label'    => __( 'Debug mode', 'dlm-captcha' ),
					'cb_label' => __( 'Enable', 'dlm-captcha' ),
					'desc'     => __( 'If enabled, your website will display and stop on the Google reCAPTCHA error.', 'dlm-captcha' ),
					'type'     => 'checkbox'
				),
			)
		);

		return $settings;
	}

	/**
	 * Static getter for sitekey
	 *
	 * @return mixed
	 */
	public static function get_sitekey() {
		return get_option( 'dlm_ca_sitekey', apply_filters( 'dlm_ca_sitekey', '' ) );
	}

	/**
	 * Static getter for sitesecret
	 *
	 * @return string
	 */
	public static function get_sitesecret() {
		return get_option( 'dlm_ca_sitesecret', apply_filters( 'dlm_ca_sitesecret', '' ) );
	}

	/**
	 * Static getter for unlock_text
	 *
	 * @return string
	 */
	public static function get_unlock_text() {
		return get_option( 'dlm_ca_unlock_text',  apply_filters( 'dlm_ca_unlock_text', 'Please complete the captcha to download the file.' ) );
	}

	/**
	 * Static getter for global
	 *
	 * @return string
	 */
	public static function is_global() {
		return ( '1' == get_option( 'dlm_ca_global', apply_filters( 'dlm_ca_global', 0 ) ) );
	}

	/**
	 * Static getter for debug
	 *
	 * @return string
	 */
	public static function is_debug() {
		return ( '1' == get_option( 'dlm_ca_debug', apply_filters( 'dlm_ca_debug', 0 ) ) );
	}

}
