<?php

if ( ! defined( 'ABSPATH' ) ) {
	exit;
} // Exit if accessed directly

class DLM_CA_Shortcode {

	private static $js_printed = false;

	public function setup() {
		// Register the shortcode
		add_shortcode( 'dlm_captcha', array( $this, 'content' ) );

		// add shortcode form the no access page
		add_action( 'dlm_no_access_after_message', array( $this, 'add_form_to_no_access_page' ) );
	}

	/**
	 * The dlm_captcha shortcode
	 *
	 * @return string
	 */
	public function content( $atts ) {

		$atts = wp_parse_args( $atts, array(
			'id'         => null,
			'version_id' => null,
			'version'    => null,
			'template'   => dlm_get_default_download_template()
		) );

		// Return nothing if no ID is set
		if ( null === $atts['id'] ) {
			return '';
		}

		// try to fetch download object for given ID
		try {
			/** @var \DLM_Download $download */
			$download = download_monitor()->service( 'download_repository' )->retrieve_single( absint( $atts['id'] ) );

			// check if version is set
			if ( null !== $atts['version'] ) {
				$atts['version_id'] = $download->get_version_id_version_name( $atts['version'] );
			}

			// check if version ID is set
			if ( null !== $atts['version_id'] ) {
				try {
					$version = download_monitor()->service( 'version_repository' )->retrieve_single( $atts['version_id'] );
					$download->set_version( $version );
				} catch ( Exception $e ) {
				}
			}

		} catch ( Exception $exception ) {
			// no download found with given ID, return empty string
			return '';
		}

		// Check if the Download is captcha locked
		if ( 'yes' !== get_post_meta( $download->get_id(), DLM_CA_Constants::META_CAPTCHA_LOCKED_KEY, true ) && ! DLM_CA_Settings::is_global() ) {
			return '';
		}

		// Put Email Lock AJAX if not done yet
		if ( false === self::$js_printed ) {
			add_action( 'wp_footer', array( $this, 'print_js' ) );
			self::$js_printed = true;
		}


		// Alright, all good. Load the template.
		ob_start();

		// Template handler
		$template_handler = new DLM_Template_Handler();

		// Load template
		$template_handler->get_template_part( 'download-captcha', $atts['template'], plugin_dir_path( DLM_Captcha::get_plugin_file() ) . 'templates/', array(
			'download'    => $download,
			'sitekey'     => DLM_CA_Settings::get_sitekey(),
			'unlock_text' => DLM_CA_Settings::get_unlock_text()
		) );

		return ob_get_clean();
	}

	/**
	 * Add dlm_captcha to no access page
	 *
	 * @param DLM_Download $download
	 *
	 * @return string
	 */
	public function add_form_to_no_access_page( $download ) {
		if ( 'yes' == get_post_meta( $download->get_id(), DLM_CA_Constants::META_CAPTCHA_LOCKED_KEY, true ) || DLM_CA_Settings::is_global() ) {
			echo $this->content( array( 'id' => $download->get_id() ) );
		}
	}

	/**
	 * Print the JS
	 */
	public function print_js() {
		?>
        <script src='https://www.google.com/recaptcha/api.js'></script>
		<?php
	}

}
