<?php
/*
	Plugin Name: Download Monitor - Captcha
	Plugin URI: https://www.download-monitor.com/
	Description: The Captcha extension for Download Monitor allows you to require users to complete a reCAPTCHA before they gain access to a download.
	Version: 4.2.0
	Author: Never5
	Author URI: http://www.never5.com/
	License: GPL v3
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

class DLM_Captcha {

	const VERSION = '4.2.0';

	public function __construct() {

		// Download Access Manager
		$access_manager = new DLM_CA_Access_Manager();
		$access_manager->setup();

		// Register Shortcode
		$shortcode = new DLM_CA_Shortcode();
		$shortcode->setup();

		// Admin only classes
		if ( is_admin() ) {

			// Download Option
			$download_option = new DLM_CA_Download_Option();
			$download_option->setup();

			// add settings
			add_filter( 'download_monitor_settings', array( 'DLM_CA_Settings', 'add_settings' ) );
		}

		// Register Extension
		add_filter( 'dlm_extensions', array( $this, 'register_extension' ) );
	}

	/**
	 * Get the plugin file
	 *
	 * @static
	 *
	 * @return String
	 */
	public static function get_plugin_file() {
		return __FILE__;
	}

	/**
	 * Register this extension
	 *
	 * @param array $extensions
	 *
	 * @return array $extensions
	 */
	public function register_extension( $extensions ) {

		$extensions[] = array(
			'file'    => 'dlm-captcha',
			'version' => DLM_Captcha::VERSION,
			'name'    => 'Captcha'
		);

		return $extensions;
	}

}

require_once dirname( __FILE__ ) . '/vendor/autoload_52.php';

function _dlm_captcha() {
	new DLM_Captcha();
}

add_action( 'plugins_loaded', '_dlm_captcha' );
