<?php

if ( ! defined( 'ABSPATH' ) ) {
	exit;
} // Exit if accessed directly

class DLM_EL_CSV_Exporter {

	/**
	 * Construct
	 */
	public function __construct() {
		add_action( 'admin_init', array( $this, 'catch_request' ), 1 );
	}

	/**
	 * Catch the request
	 */
	public function catch_request() {
		if ( isset( $_GET['dlm-el-export-csv'] ) && isset( $_GET['post-id'] ) ) {
			$this->export_csv( absint( $_GET['post-id'] ) );
		}
	}

	/**
	 * Export the CSV file
	 *
	 * @param $post_id
	 */
	private function export_csv( $post_id ) {

		// must be int
		$post_id = absint( $post_id );

		if ( $post_id > 0 ) {
			// Get Post
			$post = get_post( $post_id );

			// Get email addresses
			$email_addresses = get_post_meta( $post->ID, DLM_EL_Constants::META_EMAIL_ADDRESS );

			// csv file name
			$csv_file_name = $post->post_name;

			// Header 'row'
			$email_csv_string = 'email' . PHP_EOL;

			// Add email addresses
			$email_csv_string .= implode( PHP_EOL, $email_addresses );
		} else {

			// we need the $wpdb object
			global $wpdb;

			// get all email address
			$email_rows = $wpdb->get_results( $wpdb->prepare( 'SELECT `post_id`, `meta_value` FROM `' . $wpdb->postmeta . '` WHERE `meta_key` = %s', DLM_EL_Constants::META_EMAIL_ADDRESS ) );

			// csv file name
			$csv_file_name = 'dlm-email-lock-all';

			$email_csv_string = 'ID,title,email' . PHP_EOL;
			if ( ! empty( $email_rows ) ) {

				// loop and add
				foreach ( $email_rows as $email_row ) {
					$email_csv_string .= $email_row->post_id . ',"' . get_the_title( $email_row->post_id ) . '",' . $email_row->meta_value . PHP_EOL;
				}

			}
		}

		// CSV headers
		header( "Content-type: text/csv" );
		header( "Content-Disposition: attachment; filename=" . $csv_file_name . "-email-addresses-export.csv" );
		header( "Pragma: no-cache" );
		header( "Expires: 0" );

		// Echo CSV string
		echo $email_csv_string;

		// Done
		exit;

	}


}
