<?php

if ( ! defined( 'ABSPATH' ) ) {
	exit;
} // Exit if accessed directly

class DLM_EL_Email_Notification {

	/**
	 * Setup integration
	 */
	public function setup() {
		// add our email field
		add_filter( 'dlm_en_available_fields', array( $this, 'add_field_to_available_fields' ), 10, 1 );

		// set our field in email
		add_filter( 'dlm_en_field_el_email', array( $this, 'set_field' ), 10, 2 );
	}

	/**
	 * Add our field to available fields
	 *
	 * @param array $fields
	 *
	 * @return array
	 */
	public function add_field_to_available_fields( $fields ) {
		$fields[] = 'el_email';

		return $fields;
	}

	/**
	 * Set our email lock field in the email. This is where we're basically 'replacing' the tag.
	 *
	 * @param array $field
	 * @param DLM_Log_Item $log_item
	 *
	 * @return array
	 */
	public function set_field( $field, $log_item ) {

		// set label
		$field['label'] = __( 'Email used to unlock', 'dlm-email-lock' );

		// set el_email if set in meta
		$meta = $log_item->get_meta_data();
		if ( ! empty( $meta ) && isset( $meta['el_email'] ) ) {
			$field['value'] = $meta['el_email'];
		}else {
			$field['value'] = "-";
		}

		return $field;
	}
}