<?php

if ( ! defined( 'ABSPATH' ) ) {
	exit;
} // Exit if accessed directly

class DLM_EL_Meta_Box_Email_Addresses {

	public function __construct() {

		// Add emails meta box
		add_action( 'add_meta_boxes', array( $this, 'add_meta_box' ), 11 );

		// CSS for meta box
		add_action( 'admin_head', array( $this, 'admin_css' ) );

		// JS for meta box
		add_action( 'admin_enqueue_scripts', array( $this, 'enqueue_scripts' ) );

		// Delete email address AJAX call
		add_action( 'wp_ajax_dlm_el_delete_email', array( $this, 'ajax_delete_email' ) );
	}

	/**
	 * Add meta box
	 */
	public function add_meta_box() {
		global $post;

		// Only add for email locked downloads
		if ( 'yes' !== get_post_meta( $post->ID, DLM_EL_Constants::META_EMAIL_LOCKED_KEY, true ) ) {
			return;
		}

		// Add meta box
		add_meta_box( 'dlm-el-email-addresses', __( 'Email Addresses', 'dlm-email-lock' ), array(
			$this,
			'content'
		), 'dlm_download', 'normal', 'high' );
	}

	/**
	 * The meta box content
	 */
	public function content() {
		global $post;

		$email_addresses = get_post_meta( $post->ID, DLM_EL_Constants::META_EMAIL_ADDRESS );

		if ( count( $email_addresses ) > 0 ) {
			echo '<input type="hidden" id="dlm-el-ad-nonce" value="' . wp_create_nonce( 'dlm-el-ad-nonce' ) . '" />' . PHP_EOL;
			echo '<input type="hidden" id="dlm-el-ad-post-id" value="' . $post->ID . '" />' . PHP_EOL;
			echo '<a href="javascript:;" class="button dlm-el-export-btn" data-export-url="' . admin_url( sprintf( '?dlm-el-export-csv=1&post-id=%s', $post->ID ) ) . '">' . __( 'Export CSV', 'dlm-email-lock' ) . '</a>' . PHP_EOL;
			echo "<table class='wp-list-table widefat fixed pages dlm_el_email_addresses'>\n";
			echo "<tbody>\n";

			foreach ( $email_addresses as $email_address ) {
				echo '<tr data-email="' . $email_address . '">';
				echo '<td>' . $email_address . '</td>';
				echo '<td style="width: 20%;text-align: right;"><a href="javascript:;" class="delete">' . __( 'Delete', 'dlm-email-lock' ) . '</a></td>';
				echo '</tr>' . PHP_EOL;
			}

			echo "</tbody>\n";
			echo "</table>\n";
		} else {
			echo '<p>' . __( 'No email addresses yet.', 'dlm-email-lock' ) . '</p>';
		}
	}

	/**
	 * Meta box CSS
	 */
	public function admin_css() {
		echo '<style type = "text/css" >.dlm_el_email_addresses tr:nth-child( odd ){background - color: #FCFCFC;}.dlm_el_email_addresses tr td{padding: 20px;}.dlm-el-export-btn{float:right;margin-bottom: 10px !important;}</style>' . PHP_EOL;
	}

	/**
	 * Enqueue scripts
	 */
	public function enqueue_scripts() {
		global $pagenow;

		if ( $pagenow === 'post.php' ) {

			// Enqueue Edit Post JS
			wp_enqueue_script(
				'dlm_el_edit_post',
				plugins_url( '/assets/js/dlm-email-lock' . ( ( ! SCRIPT_DEBUG ) ? '.min' : '' ) . '.js', DLM_Email_Lock::get_plugin_file() ),
				array( 'jquery' ),
				DLM_Email_Lock::VERSION
			);

			// Make JavaScript strings translatable
			wp_localize_script( 'dlm_el_edit_post', 'dlm_el_ep_strings', array(
				'confirm_delete' => __( 'Are you sure you want to delete this email address?', 'dlm-email-lock' ),
				'delete_error'   => __( "Couldn't delete email address", 'dlm-email-lock' )
			) );


		}

	}

	/**
	 * AJAX delete email
	 */
	public function ajax_delete_email() {

		// Check AJAX nonce
		check_ajax_referer( 'dlm-el-ad-nonce', 'nonce' );

		// Vars
		$post_id = absint( $_POST['post_id'] );
		$email   = sanitize_email( $_POST['email'] );

		// Delete post meta
		$result = delete_post_meta( $post_id, DLM_EL_Constants::META_EMAIL_ADDRESS, $email );

		// Send JSON response
		wp_send_json( array( 'success' => $result ) );

		exit;
	}

}
