<?php

if ( ! defined( 'ABSPATH' ) ) {
	exit;
} // Exit if accessed directly

class DLM_EL_Page_Addon {

	private static $js_printed = false;

	public function setup() {
		// Hijack the page addon download button
		add_filter( 'dlm_page_addon_download_button', array( $this, 'page_addon_download_button' ), 10, 2 );
	}

	/**
	 * Hijack the [download] shortcode
	 *
	 * @param $content
	 * @param $download_id
	 *
	 * @return string
	 */
	public function page_addon_download_button( $content, $download_id ) {

		// access manager
		$access_manager = new DLM_EL_Access_Manager();

		try {
			$download = download_monitor()->service( 'download_repository' )->retrieve_single( $download_id );

			// only replace button with form if user has no access to download
			if ( false == $access_manager->check_access( true, $download, null ) ) {

				// Use the callback for the 'dlm_email_lock' shortcode
				$hijacked_content = $this->content( array( 'id' => $download_id ) );

				// Replace content if we've got content
				if ( '' !== $hijacked_content ) {
					$content = $hijacked_content;
				}

			}
		} catch ( Exception $exception ) {
			// no download found
		}


		// Return content
		return $content;
	}

	/**
	 * The dlm_email_lock shortcode
	 *
	 * @return string
	 */
	public function content( $atts ) {

		$atts = wp_parse_args( $atts, array(
			'id'       => null,
			'template' => dlm_get_default_download_template()
		) );

		// Return nothing if no ID is set
		if ( null === $atts['id'] ) {
			return '';
		}

		try {
		    /** @var DLM_Download $download */
			$download = download_monitor()->service( 'download_repository' )->retrieve_single( absint( $atts['id'] ) );
		} catch ( Exception $exception ) {
			// no download found
			return '';
		}

		// Check if the Download is email locked
		if ( 'yes' !== get_post_meta( $download->get_id(), DLM_EL_Constants::META_EMAIL_LOCKED_KEY, true ) ) {
			return '';
		}

		// Put Email Lock AJAX if not done yet
		if ( false === self::$js_printed ) {
			add_action( 'wp_footer', array( $this, 'print_js' ) );
			self::$js_printed = true;
		}


		// Alright, all good. Load the template.
		ob_start();

		// Template handler
		$template_handler = new DLM_Template_Handler();

		// Load template
		$template_handler->get_template_part( 'download-email-lock', $atts['template'], plugin_dir_path( DLM_Email_Lock::get_plugin_file() ) . 'templates/', array(
			'download' => $download
		) );

		return ob_get_clean();
	}

	/**
	 * Print the email lock styles
	 */
	public function print_styles() {
		echo '<style type="text/css" media="screen">.dlm-email-lock input.dlm-email-lock-email{width:70%;float:left}input.dlm-email-lock-email.error{border:1px solid #ff0000;border-right:0}.dlm-email-lock .dlm-email-lock-submit{width:30%} }</style>' . PHP_EOL;
	}

	/**
	 * Print the JS
	 */
	public function print_js() {
		?>
        <script type="text/javascript">
			jQuery( function ( $ ) {
				$.each( $( '.dlm-email-lock' ), function ( k, v ) {
					$( v ).submit( function () {
						var email_addr = $( v ).find( '.dlm-email-lock-email' );
						var email_submit = $( v ).find( '.dlm-email-lock-submit' );
						var email_regex = /^[a-z0-9._%+-]+@[a-z0-9.-]+\.[a-z]{2,4}$/;
						if ( true === email_regex.test( email_addr.val() ) ) {
							email_addr.removeClass( 'error' );
							email_submit.attr( 'disabled', true );
							return true;
						} else {
							email_addr.addClass( 'error' );
							return false;
						}
						return false;
					} );
				} )
			} );
        </script>
		<?php
	}

}