<?php

class DLM_EL_Settings {

	/**
	 * Add settings to DLM settings
	 *
	 * @param $settings
	 *
	 * @return mixed
	 */
	public static function add_settings( $settings ) {

		$settings['email_lock'] = array(
			__( 'Email Lock', 'dlm-email-lock' ),
			array(
				array(
					'name'    => 'dlm_el_unlock_type',
					'std'     => '',
					'label'   => __( 'Submitting email unlocks', 'dlm-email-lock' ),
					'desc'    => __( 'Select what should be unlocked upon email address submission. Only applies to Email Lock locked downloads.', 'dlm-email-lock' ),
					'type'    => 'select',
					'options' => array(
						'per_download' => __( 'only requested download', 'dlm-email-lock' ),
						'global'       => __( 'all locked downloads', 'dlm-email-lock' ),
					)
				),
				array(
					'name'    => 'dlm_el_optin',
					'std'     => '',
					'label'   => __( 'Opt-in field ', 'dlm-email-lock' ),
					'cb_label'   => __( 'Add checkbox ', 'dlm-email-lock' ),
					'desc'    => __( 'Enter your opt-in text. If you enter any text here, a checkbox will be added to your Email Lock forms that your users are required to check. If this field is left empty, no opt-in checkbox will be added.', 'dlm-email-lock' ),
					'type'    => 'textarea',
				),
			)
		);

		// check if Download Monitor 4.0 or higher is installed so we can add custom fields
		if ( version_compare( DLM_VERSION, '4.0.0', '>=' ) ) {

			// add new custom field to settings array
			$settings['email_lock'][1][] = array(
				'name'  => 'dlm_el_export_link',
				'std'   => '',
				'label' => __( 'Export All', 'dlm-email-lock' ),
				'desc'  => __( 'Export all collected email addresses of all downloads in one CSV file', 'dlm-email-lock' ),
				'type'  => 'export_all_link',
			);

			// add new field object
			add_filter( 'dlm_setting_field_export_all_link', array( __CLASS__, 'add_custom_field' ), 10, 4 );


		}

		return $settings;
	}

	/**
	 * @param DLM_Admin_Fields_Field $field
	 * @param array $option
	 * @param String $value
	 * @param String $placeholder
	 *
	 * @return mixed
	 */
	public static function add_custom_field( $field, $option, $value, $placeholder ) {

		// check if this is our exact field
		if ( 'dlm_el_export_link' === $option['name'] ) {
			$field = new DLM_EL_Field_Export_All_Link( $option['name'], $value, $placeholder );
		}

		return $field;
	}

}