<?php

if ( ! defined( 'ABSPATH' ) ) {
	exit;
} // Exit if accessed directly

class DLM_EL_Shortcode_Email_Lock {

	private static $js_printed = false;

	public function setup() {
		// Register the shortcode
		add_shortcode( 'dlm_email_lock', array( $this, 'content' ) );

		// add shortcode form the no access page
		add_action( 'dlm_no_access_after_message', array( $this, 'add_form_to_no_access_page' ) );

		// WP frontend styling
		add_action( 'wp_print_styles', array( $this, 'print_styles' ), 10 );
	}

	/**
	 * The dlm_email_lock shortcode
	 *
	 * @return string
	 */
	public function content( $atts ) {

		$atts = wp_parse_args( $atts, array(
			'id'       => null,
			'template' => dlm_get_default_download_template()
		) );

		// Return nothing if no ID is set
		if ( null === $atts['id'] ) {
			return '';
		}

		try {
			/** @var DLM_Download $download */
			$download = download_monitor()->service( 'download_repository' )->retrieve_single( absint( $atts['id'] ) );
		} catch ( Exception $exception ) {
			// no download found
			return '';
		}

		// Check if the Download is email locked
		if ( 'yes' !== get_post_meta( $download->get_id(), DLM_EL_Constants::META_EMAIL_LOCKED_KEY, true ) ) {
			return '';
		}

		// Put Email Lock AJAX if not done yet
		if ( false === self::$js_printed ) {
			add_action( 'wp_footer', array( $this, 'print_js' ) );
			self::$js_printed = true;
		}

		// check if there is an opt-in text
		$optin         = trim( get_option( 'dlm_el_optin', '' ) );
		$base_template = 'download-email-lock';
		if ( ! empty( $optin ) ) {
			$base_template = 'download-email-lock-optin';
		}

		// Alright, all good. Load the template.
		ob_start();

		// Template handler
		$template_handler = new DLM_Template_Handler();

		// Load template
		$template_handler->get_template_part( $base_template, $atts['template'], plugin_dir_path( DLM_Email_Lock::get_plugin_file() ) . 'templates/', array(
			'download' => $download,
			'optin'    => $optin
		) );

		return ob_get_clean();
	}

	/**
	 * Add email lock shortcode to no access page
	 *
	 * @param DLM_Download $download
	 *
	 * @return string
	 */
	public function add_form_to_no_access_page( $download ) {

		if ( 'yes' == get_post_meta( $download->get_id(), DLM_EL_Constants::META_EMAIL_LOCKED_KEY, true ) ) {
			echo $this->content( array( 'id' => $download->get_id() ) );
		}
	}

	/**
	 * Print the email lock styles
	 */
	public function print_styles() {
		echo '<style type="text/css" media="screen">.dlm-email-lock input.dlm-email-lock-email{width:70%;float:left}input.dlm-email-lock-email.error{border:1px solid #ff0000;}.dlm-email-lock .dlm-email-lock-submit{width:30%}.dlm-email-lock-optin p {display: inline-block;width: 100%;margin: 0;padding: 0.5em 0;}.dlm-email-lock-optin-field-optin label {padding: .5em;cursor: pointer;font-size:90%;}.dlm-email-lock-optin-field-optin label span {padding: 0 0 0 .5em;}.dlm-email-lock-optin-field-optin label.error{border:1px solid #ff0000;}</style>' . PHP_EOL;
	}

	/**
	 * Print the JS
	 */
	public function print_js() {
		?>
        <script type="text/javascript">
	        jQuery( function ( $ ) {
		        $.each( $( '.dlm-email-lock' ), function ( k, v ) {
			        $( v ).submit( function () {
				        // store if we ran into errors
				        var has_errors = false;

				        // vars
				        var email_addr = $( v ).find( '.dlm-email-lock-email' );
				        var email_optin = $( v ).find( '.dlm-email-lock-email-optin' );
				        var email_submit = $( v ).find( '.dlm-email-lock-submit' );
				        var email_regex = /^[a-z0-9._%+-]+@[a-z0-9.-]+\.[a-z]{2,5}$/i;

				        // remove previous errors
				        email_addr.removeClass( 'error' );
				        email_optin.closest( 'label' ).removeClass( 'error' );

				        // check email
				        if ( false === email_regex.test( email_addr.val() ) ) {
					        email_addr.addClass( 'error' );
					        has_errors = true;
				        }

				        // check if consent is required and if so, if it is given
				        if ( email_optin.length > 0 && ! email_optin.is(":checked") ) {
					        email_optin.closest( 'label' ).addClass( 'error' );
					        has_errors = true;
				        }

				        // submit if there are no errors
				        if ( false === has_errors ) {
					        email_submit.attr( 'disabled', true );
					        return true;
				        }

				        // default succes is false
				        return false;
			        } );
				} )
			} );
        </script>
		<?php
	}

}