<?php
/*
	Plugin Name: Download Monitor - Email Lock
	Plugin URI: https://www.download-monitor.com/
	Description: The Email Lock extension for Download Monitor allows you to require users to fill in their email address before they gain access to a download.
	Version: 4.1.1
	Author: Never5
	Author URI: http://www.never5.com/
	License: GPL v3
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

class DLM_Email_Lock {

	const VERSION = '4.1.1';

	public function __construct() {

		// Download Access Manager
		$access_manager = new DLM_EL_Access_Manager();
		$access_manager->setup();

		// Download Log Manager
		$log_manager = new DLM_EL_Log_Manager();
		$log_manager->setup();

		// Register Shortcode
		$shortcode = new DLM_EL_Shortcode_Email_Lock();
		$shortcode->setup();

		// Page Addon compatibility
		$page_addon_compat = new DLM_EL_Page_Addon();
		$page_addon_compat->setup();

		// Email Notification Integration
		$email_notification_integration = new DLM_EL_Email_Notification();
		$email_notification_integration->setup();

		// Admin only classes
		if ( is_admin() ) {

			// Download Option
			$download_option = new DLM_EL_Download_Option();
			$download_option->setup();

			// add export all page for Download Monitor versions lower than 4.0
			if ( version_compare( DLM_VERSION, '4.0.0', '<' ) ) {
				$export_all_page = new DLM_EL_Export_All_Page();
				$export_all_page->setup();
			}

			// Email Addresses meta box
			new DLM_EL_Meta_Box_Email_Addresses();

			// CSV exporter
			new DLM_EL_CSV_Exporter();

			// add settings
			add_filter( 'download_monitor_settings', array( 'DLM_EL_Settings', 'add_settings' ) );
		}

		// Register Extension
		add_filter( 'dlm_extensions', array( $this, 'register_extension' ) );
	}

	/**
	 * Get the plugin file
	 *
	 * @static
	 *
	 * @return String
	 */
	public static function get_plugin_file() {
		return __FILE__;
	}

	/**
	 * Register this extension
	 *
	 * @param array $extensions
	 *
	 * @return array $extensions
	 */
	public function register_extension( $extensions ) {

		$extensions[] = array(
			'file'    => 'dlm-email-lock',
			'version' => self::VERSION,
			'name'    => 'Email Lock'
		);

		return $extensions;
	}

}

require_once dirname( __FILE__ ) . '/vendor/autoload_52.php';

function _dlm_email_lock() {
	new DLM_Email_Lock();
}

add_action( 'plugins_loaded', '_dlm_email_lock' );