<?php

class DLM_GD_Handler {

	/**
	 * Replace Google Drive URL in file path
	 *
	 * @param DLM_Download $download
	 * @param DLM_Download_Version $version
	 * @param string $file_path
	 *
	 */
	public function handle( $download, $version, $file_path ) {

		// check if Google Drive URL
		if ( stristr( $file_path, 'drive.google.com/file/' ) ) {

			// convert
			$file_path = $this->convert( $file_path );

			// new DLM Logging object
			$logging = new DLM_Logging();

			// Check if logging is enabled
			if ( $logging->is_logging_enabled() ) {

				// setup new log item object
				$log_item = new \DLM_Log_Item();
				$log_item->set_user_id( absint( get_current_user_id() ) );
				$log_item->set_user_ip( \DLM_Utils::get_visitor_ip() );
				$log_item->set_user_agent( \DLM_Utils::get_visitor_ua() );
				$log_item->set_download_id( absint( $download->get_id() ) );
				$log_item->set_version_id( absint( $version->get_id() ) );
				$log_item->set_version( $version->get_version() );
				$log_item->set_download_date( new \DateTime( current_time( 'mysql' ) ) );
				$log_item->set_download_status( 'redirected' );
				$log_item->set_download_status_message( __( 'Redirected to Google Drive', 'download_monitor' ) );

				// persist log item
				download_monitor()->service( 'log_item_repository' )->persist( $log_item );
			}
			
			// redirect to correct Google Drive URL
			header( 'Location: ' . $file_path );

			// done
			exit;

		}

	}

	/**
	 * Convert Google Drive $file_path
	 *
	 * @param string $file_path
	 *
	 * @return string
	 */
	private function convert( $file_path ) {
		if ( preg_match( '`https?\://drive.google.com/file/d/([0-9A-z\_\-]+)`', $file_path, $matches ) ) {
			$file_path = "https://drive.google.com/uc?export=download&id=" . $matches[1];
		}

		return $file_path;
	}

}