<?php

namespace DLM\MailChimp;

class Admin {

	const SETTINGS_CAP = 'manage_downloads';

	/**
	 * Constructor
	 */
	public function __construct() {	}

	/**
	 * Add hooks
	 */
	public function add_hooks() {
		add_action( 'dlm_options_end', array( $this, 'show_settings' ) );
		add_action( 'dlm_save_meta_boxes', array( $this, 'save_settings' ) );
	}

	/**
	 * Add settings to the Download settings.
	 *
	 * @param $post_id
	 */
	public function show_settings( $post_id ) {
		$locked = get_post_meta( $post_id, '_mailchimp_locked', true );
		$selected_list_id = get_post_meta( $post_id, '_mailchimp_list_id', true );
		$selected_form_id = get_post_meta( $post_id, '_mailchimp_form_id', true );
		$mailchimp = new \MC4WP_MailChimp();

		$forms = mc4wp_get_forms();
		?>
		<div class="form-field">
			<p class="form-field-checkbox">
				<label for="_mailchimp_locked">
					<input type="checkbox" name="_mailchimp_locked" id="_mailchimp_locked" value="yes" <?php checked( $locked, 'yes' ); ?> />
					<?php _e( 'Subscribers only', 'dlm-mailchimp-lock' ); ?>
				</label>

				<span class="dlm-description" style="display: block;"><?php _e( 'Only MailChimp subscribers will be able to access the file via a download link if this is enabled.', 'dlm-mailchimp-lock' ); ?></span>

				<span style="display: <?php echo ( $locked === 'yes' ) ? 'block' : 'none'; ?>;" id="_dlm_mailchimp_settings">
					<label style="font-weight: bold; display: block; margin: 10px 0 0;">MailChimp list</label>
					<select name="_mailchimp_list_id">
						<option disabled><?php esc_html_e( 'MailChimp list', 'dlm-mailchimp-lock' ); ?></option>
						<?php foreach( $mailchimp->get_lists() as $list ) { ?>
							<option value="<?php echo esc_attr( $list->id ); ?>" <?php selected( $list->id, $selected_list_id ); ?>>
								<?php echo esc_html( $list->name ); ?>
							</option>
						<?php } ?>
					</select>

					<label style="font-weight: bold; display: block; margin: 10px 0 0;">Form to show on "No Access" page</label>
					<select name="_mailchimp_form_id">
						<option value="0"><?php esc_html_e( 'Do not show a form', 'dlm-mailchimp-lock' ); ?></option>

						<optgroup label="<?php esc_attr_e( 'Forms', 'mailchimp-for-wp' ); ?>">
						<?php foreach( $forms as $form ) {
							echo sprintf( '<option value="%d" %s>%s</option>', $form->ID, selected( $selected_form_id, $form->ID, false), $form->name );
						} ?>
						</optgroup>
					</select>

					<?php if( ! empty( $selected_form_id ) ) {
						echo '<span class="dlm-description" style="font-style: italic;">';
						echo sprintf( '<a href="%s">' . __( 'Click here to edit the selected form.', 'dlm-mailchimp-lock' ) . '</a>', mc4wp_get_edit_form_url( $selected_form_id ) );
						echo '</span>';
					} ?>
				</span>
			</p>
		</div>
		<?php

		// print js snippet in footer
		add_action( 'admin_footer', array( $this, 'javascript' ) );
	}

	/**
	 * @param $post_id
	 */
	public function save_settings( $post_id ) {
		$locked = ( isset( $_POST['_mailchimp_locked'] ) && $_POST['_mailchimp_locked'] == 'yes' ) ? 'yes' : 'no';
		$list_id = ( isset( $_POST['_mailchimp_list_id'] ) ) ? sanitize_text_field( $_POST['_mailchimp_list_id'] ) : '';
		$form_id = ( isset( $_POST['_mailchimp_form_id'] ) ) ? sanitize_text_field( $_POST['_mailchimp_form_id'] ) : '';

		if( 'yes' === $locked ) {
			update_post_meta( $post_id, '_mailchimp_locked', $locked );
			update_post_meta( $post_id, '_mailchimp_list_id', $list_id );
			update_post_meta( $post_id, '_mailchimp_form_id', $form_id );
		} else {
			delete_post_meta( $post_id, '_mailchimp_locked' );
			delete_post_meta( $post_id, '_mailchimp_list_id' );
			delete_post_meta( $post_id, '_mailchimp_form_id' );
		}
	}

	public function javascript() {
		?>
		<script type="text/javascript">
			(function() {
				var triggerEl = document.getElementById('_mailchimp_locked');
				var targetEl = document.getElementById('_dlm_mailchimp_settings');

				// for IE8 compatibility, use a simple `onchange`
				triggerEl.onchange = function() {
					targetEl.style.display = ( this.checked ) ? 'block' : 'none';
				};

			})();
		</script>
		<?php
	}

	/**
	 * @param $url
	 *
	 * @return string
	 */
	protected function asset_url( $url ) {
		return plugins_url( '/assets' . $url, DLM_MAILCHIMP_LOCK_FILE );
	}

}