<?php

if ( ! defined( 'ABSPATH' ) ) {
	exit;
} // Exit if accessed directly

class DLM_NF_Access_Manager {

	/**
	 * Setup class
	 */
	public function setup() {
		// Filter the download to redirect regular download buttons of mail locked downloads
		add_filter( 'dlm_can_download', array( $this, 'check_access' ), 30, 2 );
	}

	/**
	 * Check if requester has access to download
	 *
	 * @param $has_access
	 * @param $download
	 *
	 * @return bool
	 */
	public function check_access( $has_access, $download ) {

		// Check if the Download is email locked
		if ( '' === $download->{DLM_NF_Constants::META_FORM} || 0 == $download->{DLM_NF_Constants::META_FORM} ) {
			return $has_access;
		}

		// get unlock type
		$unlock_type = $this->get_unlock_type();

		// This is Ninja Form Download, set access default to false
		$has_access = false;

		// Check if COOKIE is set
		if ( isset( $_COOKIE[ DLM_NF_Constants::COOKIE ] ) ) {

			// JSON decode cookie data
			$cookie_data = json_decode( stripslashes( $_COOKIE[ DLM_NF_Constants::COOKIE ] ), true );

			// Check if decode went ok
			if ( null !== $cookie_data ) {

				// check for global access
				if ( 'global' === $unlock_type ) {

					// GF cookie exists and the unlock_type is set to global, grant access
					$has_access = true;
				}

				// Check cookie data
				if ( false === $has_access && $cookie_data['form_id'] == $download->{DLM_NF_Constants::META_FORM} ) {

					// check if unlock type is set per_form
					if ( 'per_form' === $unlock_type ) {
						// NF cookie exists and the form_id matches this form ID
						$has_access = true;
					}

					if ( false === $has_access && $cookie_data['download_id'] == $download->id ) {

						// Form ID and Download ID match, check if there's a submission with this id
						$submission = get_post( absint( $cookie_data['sub_id'] ) );

						// Check submission
						if ( null !== $submission ) {

							// Check Post Type
							if ( 'nf_sub' === $submission->post_type ) {

								// Check Download Hash
								$download_hash = get_post_meta( $submission->ID, DLM_NF_Constants::META_SUB_DOWNLOAD_HASH, true );
								if ( '' !== $download_hash && isset( $cookie_data['download_hash'] ) && $download_hash === $cookie_data['download_hash'] ) {

									// There it is, you've got access. Enjoy the file :)
									$has_access = true;
								}

							}
						}
					}
				}

			}
		}

		return $has_access;
	}

	/**
	 * Get unlock type setting
	 *
	 * @return string
	 */
	public function get_unlock_type() {
		return get_option( 'dlm_nf_unlock_type', 'per_download' );
	}

}