<?php

if ( ! defined( 'ABSPATH' ) ) {
	exit;
} // Exit if accessed directly

class DLM_NF_Meta_Box_Ninja_Forms {

	public function __construct() {

		// Add meta box
		add_action( 'add_meta_boxes', array( $this, 'add_meta_box' ) );

		// Save meta box
		add_action( 'dlm_save_meta_boxes', array( $this, 'save' ) );

		// CSS for meta box
		add_action( 'admin_head', array( $this, 'admin_css' ) );
	}

	/**
	 * Add meta box
	 */
	public function add_meta_box() {

		// Add meta box
		add_meta_box( 'dlm-nf-ninja-forms', __( 'Ninja Forms Form', 'dlm-ninja-forms' ), array(
			$this,
			'content'
		), 'dlm_download', 'side', 'default' );
	}

	/**
	 * The meta box content
	 */
	public function content() {
		global $post;

		?>
		<p class="dlm-nf-form-wrapper">
			<select name="<?php echo DLM_NF_Constants::META_FORM; ?>">
				<option value="0"><?php _e( 'Select Form', 'dlm-nf-form' ); ?></option>
				<?php
				$forms = ninja_forms_get_all_forms();
				$current_form_id = get_post_meta( $post->ID, DLM_NF_Constants::META_FORM, true );

				if ( count( $forms ) > 0 ) {
					foreach ( $forms as $form ) {
						echo '<option value="' . $form['id'] . '" ' . selected( $form['id'], $current_form_id, false ) . '>' . $form['name'] . '</option>' . PHP_EOL;
					}
				}
				?>
			</select>
			<small><?php _e( "After you've selected a form and save the download, the download will be 'locked' with the selected form.", 'dlm-ninja-forms' ); ?></small>
		</p>
	<?php
	}

	/**
	 * Save the meta box
	 *
	 * @param $post_id
	 */
	public function save( $post_id ) {
		if ( isset( $_POST[ DLM_NF_Constants::META_FORM ] ) ) {
			$form_id = absint( $_POST[ DLM_NF_Constants::META_FORM ] );
			if ( 0 !== $form_id ) {
				update_post_meta( $post_id, DLM_NF_Constants::META_FORM, $form_id );
			} else {
				delete_post_meta( $post_id, DLM_NF_Constants::META_FORM );
			}

		}
	}

	/**
	 * Meta box CSS
	 */
	public function admin_css() {
		?>
		<style type="text/css">.dlm-nf-form-wrapper select {width: 100%;}.dlm-nf-form-wrapper small {display: block;padding: 15px 0;}</style>
	<?php
	}

}
