<?php

if ( ! defined( 'ABSPATH' ) ) {
	exit;
} // Exit if accessed directly

class DLM_NF_Ninja_Forms_Handler {

	/**
	 *
	 */
	public function setup() {

		// Download Field
		// add_action( 'ninja_forms_display_fields', array( $this, 'add_download_id_field' ) );

		// Set entry id in cookie
		add_action( 'ninja_forms_post_process', array( $this, 'set_entry_id_cookie' ) );

		// Add download button to success message
		add_filter( 'nf_success_msg', array( $this, 'add_download_button_to_success_msg' ), 10, 2 );
	}

	/**
	 * Add the _download_id field
	 */
	public function add_download_id_field() {
		echo '<input type="hidden" name="' . DLM_NF_Constants::FIELD_DOWNLOAD_ID . '" value="%DOWNLOAD_ID%" />' . PHP_EOL;
	}

	/**
	 * Set entry id in cookie
	 */
	public function set_entry_id_cookie() {
		global $ninja_forms_processing;

		$download_id = absint( $ninja_forms_processing->get_extra_value( DLM_NF_Constants::FIELD_DOWNLOAD_ID ) );

		if ( $download_id > 0 ) {

			// Headers shouldn't be sent
			if ( ! headers_sent() ) {

				// Generate download hash
				$download_hash = md5( time() + mt_rand() + 'hallometad' );

				// Set meta
				update_post_meta( $ninja_forms_processing->data['form']['sub_id'], DLM_NF_Constants::META_SUB_DOWNLOAD_HASH, $download_hash );

				// Set the cookie
				setcookie( DLM_NF_Constants::COOKIE, json_encode( array(
					'sub_id'        => $ninja_forms_processing->data['form']['sub_id'],
					'form_id'       => $ninja_forms_processing->data['form_ID'],
					'download_id'   => $download_id,
					'download_hash' => $download_hash
				) ), 0, '/' );
			}

		}

	}

	/**
	 * Add download button to success message if this is a form send
	 *
	 * @param $msg
	 * @param $form_id
	 *
	 * @return string
	 */
	public function add_download_button_to_success_msg( $msg, $form_id ) {
		global $ninja_forms_processing;

		// Get Download ID
		$download_id = absint( $ninja_forms_processing->get_extra_value( DLM_NF_Constants::FIELD_DOWNLOAD_ID ) );

		// Check if set
		if ( false !== $download_id && '' !== $download_id && $download_id > 0 ) {

			// Get Download
			$download = new DLM_Download( $download_id );

			// template handler
			$template_handler = new DLM_Template_Handler();

			// load template
			ob_start();
			$template_handler->get_template_part( 'nf-download-link', '', plugin_dir_path( DLM_Ninja_Forms::get_plugin_file() ) . 'templates/', array( 'download' => $download ) );
			$msg .= ob_get_clean();
		}

		return $msg;
	}

}
