<?php

if ( ! defined( 'ABSPATH' ) ) {
	exit;
} // Exit if accessed directly

class DLM_TC_Access_Manager {

	/**
	 * Setup class
	 */
	public function setup() {
		add_filter( 'dlm_can_download', array( $this, 'check_access' ), 30, 3 );
	}

	/**
	 * Check if download is tc locked
	 *
	 * @param $download_id
	 *
	 * @return bool
	 */
	public static function is_tc_locked( $download_id ) {

		// get global lock option
		$global = apply_filters( 'dlm_tc_all_downloads_locked', get_option( 'dlm_tc_global', '0' ) );

		// get post meta lock
		$tc_locked = get_post_meta( $download_id, DLM_TC_Constants::META_LOCKED_KEY, true );

		return ( 'yes' == $tc_locked || '1' == $global );
	}

	/**
	 * Check if requester has access to download
	 *
	 * @param bool $has_access
	 * @param DLM_Download $download
	 * @param string $version
	 *
	 * @return bool
	 */
	public function check_access( $has_access, $download, $version ) {

		if ( ! self::is_tc_locked( $download->get_id() ) ) {
			return $has_access;
		}

		// check if the terms and conditions are checked
		if ( ! isset( $_POST['tc_accepted'] ) || '1' != $_POST['tc_accepted'] ) {
			$has_access = false;
		}

		return $has_access;
	}

}