<?php

/*
	Plugin Name: Download Monitor - Terms and Conditions
	Plugin URI: https://www.download-monitor.com/
	Description: The Terms and Conditions extension requires your users to accept your terms and conditions before they can download your files.
	Version: 4.0.0
	Author: Never5
	Author URI: http://www.never5.com/
	License: GPL v3
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

class DLM_Terms_And_Conditions {

	const VERSION = '4.0.0';

	public function __construct() {

		// Download Access Manager
		$access_manager = new DLM_TC_Access_Manager();
		$access_manager->setup();

		// add shortcode
		$shortcodes = new DLM_TC_Shortcodes();
		$shortcodes->register();

		// Admin only classes
		if ( is_admin() ) {

			// Download Option
			$download_option = new DLM_TC_Download_Option();
			$download_option->setup();

			// settings
			$options = new DLM_TC_Options();
			$options->setup();

			// admin notices
			add_action( 'admin_notices', array( 'DLM_TC_Admin_Notices', 'notices' ) );
		}

		// no access page
		add_action( 'dlm_no_access_after_message', array( $this, 'add_to_no_access_page' ) );

		// Register Extension
		add_filter( 'dlm_extensions', array( $this, 'register_extension' ) );
	}

	/**
	 * Get the plugin file
	 *
	 * @static
	 *
	 * @return String
	 */
	public static function get_plugin_file() {
		return __FILE__;
	}

	/**
	 * Register this extension
	 *
	 * @param array $extensions
	 *
	 * @return array $extensions
	 */
	public function register_extension( $extensions ) {
		$extensions[] = 'dlm-terms-and-conditions';

		return $extensions;
	}

	/**
	 * Add tweet button to no access page
	 *
	 * @param DLM_Download $download
	 */
	public function add_to_no_access_page( $download ) {

		if ( DLM_TC_Access_Manager::is_tc_locked( $download->get_id() ) ) {

			// template handler
			$template_handler = new DLM_Template_Handler();

			// enqueue shortcode JS
			DLM_TC_Assets::enqueue_shortcode_js();

			// unlock text
			$unlock_text = apply_filters( 'dlm_tc_unlock_text', get_option( 'dlm_tc_text', __( 'I accept the terms & conditions', 'dlm-terms-and-conditions' ) ), $download );

			// load template
			$template_handler->get_template_part( 'tc-form', '', plugin_dir_path( DLM_Terms_And_Conditions::get_plugin_file() ) . 'templates/', array(
				'download'    => $download,
				'unlock_text' => $unlock_text
			) );

		}

	}


}

require_once dirname( __FILE__ ) . '/vendor/autoload_52.php';

function __dlm_terms_and_conditions() {
	new DLM_Terms_And_Conditions();
}

add_action( 'plugins_loaded', '__dlm_terms_and_conditions' );