<?php

if ( ! defined( 'ABSPATH' ) ) {
	exit;
} // Exit if accessed directly

class DLM_TL_Download_Option {

	/**
	 * Setup the class
	 */
	public function setup() {

		// Add email locked download option
		add_action( 'dlm_options_end', array( $this, 'add_download_option' ), 10, 1 );

		// Save download options
		add_action( 'dlm_save_metabox', array( $this, 'save_download_option' ), 10, 1 );

	}

	/**
	 * Add twitter lock to download options
	 *
	 * @param int $post_id
	 */
	public function add_download_option( $post_id ) {

		// get if twitter locked
		$twitter_locked = get_post_meta( $post_id, DLM_TL_Constants::META_TWITTER_LOCKED_KEY, true );

		// get tweet text
		$tweet_text = get_post_meta( $post_id, DLM_TL_Constants::META_TWITTER_TWEET_TEXT, true );
	?>
		<div class="form-field form-field-checkbox">
			<input type="checkbox" name="<?php echo DLM_TL_Constants::META_TWITTER_LOCKED_KEY; ?>" id="<?php echo DLM_TL_Constants::META_TWITTER_LOCKED_KEY; ?>" <?php checked( $twitter_locked, 'yes' ); ?> />
			<label for="<?php echo DLM_TL_Constants::META_TWITTER_LOCKED_KEY; ?>"><?php _e( 'Twitter Lock', 'dlm-twitter-lock' ); ?></label>
			<span class="dlm-description"><?php _e( 'Twitter locked downloads will only be available after user tweets a predefined text.', 'dlm-twitter-lock' ); ?></span>

			<div class="dlm-tl-tweet-input-wrap<?php echo( ( 'yes' === $twitter_locked ) ? ' dlm_tweet_container-active' : '' ); ?>">
				<?php wp_nonce_field( __FILE__, '_dlm_tl_nonce' ); ?>
				<textarea class="widefat"
				          name="<?php echo DLM_TL_Constants::META_TWITTER_TWEET_TEXT; ?>"
				          id="<?php echo DLM_TL_Constants::META_TWITTER_TWEET_TEXT; ?>"
					><?php echo $tweet_text; ?></textarea>
			</div>

		</div>
	<?php
	}

	/**
	 * Save download option
	 *
	 * @param $post_id
	 */
	public function save_download_option( $post_id ) {
		$twitter_locked = ( isset( $_POST[ DLM_TL_Constants::META_TWITTER_LOCKED_KEY] ) ) ? 'yes' : 'no';
		if( 'yes' === $twitter_locked ) {

			// check nonce if download is marked as twitter locked
			if( ! isset( $_POST['_dlm_tl_nonce'] )|| ! wp_verify_nonce( $_POST['_dlm_tl_nonce'], __FILE__ ) ) {
				return;
			}

			// mark download as Twitter locked
			update_post_meta( $post_id, DLM_TL_Constants::META_TWITTER_LOCKED_KEY, $twitter_locked );

			// get tweet text
			$tweet_text = isset( $_POST[ DLM_TL_Constants::META_TWITTER_TWEET_TEXT ] ) ? $_POST[ DLM_TL_Constants::META_TWITTER_TWEET_TEXT ] : '';

			// set tweet text
			update_post_meta( $post_id, DLM_TL_Constants::META_TWITTER_TWEET_TEXT, $tweet_text );
		}else {
			delete_post_meta( $post_id, DLM_TL_Constants::META_TWITTER_LOCKED_KEY );
			delete_post_meta( $post_id, DLM_TL_Constants::META_TWITTER_TWEET_TEXT );
		}
	}

}