<?php

final class DLM_TL_Session {

	/** @var DLM_TL_Session instance */
	private static $instance = null;

	/** Cookie key constant */
	const COOKIE = 'dlm-tl-oath-session-id';

	/** @var array oauth session data */
	public $data = array();

	/**
	 * Constructor
	 */
	private function __construct() {

		// check if cookie
		if ( isset( $_COOKIE[ self::COOKIE ] ) ) {

			// load transient data into
			$transient = get_transient( $_COOKIE[ self::COOKIE ] );
			if ( false !== $transient ) {
				$this->data = $transient;
			}

		} else {

			// set cookie
			$this->set_cookie();
		}

	}

	/**
	 * Singleton getter
	 *
	 * @return DLM_TL_Session
	 */
	public static function get_instance() {
		if ( null === self::$instance ) {
			self::$instance = new self();
		}

		return self::$instance;
	}

	/**
	 * Generate oauth session id
	 *
	 * @return string
	 */
	private function generate_id() {
		// get WordPress auth salt
		$auth_salt = ( defined( 'AUTH_SALT' ) ? AUTH_SALT : '' );

		return md5( time() . mt_rand( 0, 100 ) . mt_rand( 0, 100 ) . $auth_salt );
	}

	/**
	 * Set the reference cookie
	 */
	private function set_cookie() {

		// generate ID
		$id = $this->generate_id();

		// Set the cookie
		setcookie( self::COOKIE, $id, 0, '/' );
		$_COOKIE[ self::COOKIE ] = $id;
	}

	/**
	 * Persist data
	 */
	public function persist() {
		set_transient( $_COOKIE[ self::COOKIE ], $this->data, DAY_IN_SECONDS );
	}

	/**
	 * Destroy session
	 */
	public function destroy() {
		// delete transient
		delete_transient( $_COOKIE[ self::COOKIE ] );

		// delete cookie
		@setcookie( self::COOKIE, '', time() - 3600, '/' );
	}

}